from analytics_ingest.internal.schemas.network_schema import NetworkStatsSchema


def make_network_input(config, variables):
    stats = NetworkStatsSchema.from_variables(variables, config.vehicle_id)
    return {"input": stats.model_dump()}



from .configuration import configuration_factory
from .dtc import dtc_factory
from .gps import gps_factory
from .message import message_factory
from .network_stats import network_stats_factory
from .signal import signal_factory

__all__ = [
    "configuration_factory",
    "message_factory",
    "signal_factory",
    "gps_factory",
    "dtc_factory",
    "network_stats_factory",
]



import logging

from faker import Faker

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

fake = Faker()


def configuration_factory():
    vehicle_id = fake.random_int(min=1000, max=10000000)
    device_id = fake.random_int(min=1000, max=10000000)
    organization_id = fake.random_int(min=1000, max=10000000)
    fleet_id = fake.random_int(min=1000, max=10000000)
    return {
        "vehicle_id": vehicle_id,
        "fleet_id": fleet_id,
        "organization_id": organization_id,
        "device_id": device_id,
    }



import unittest

from analytics_ingest.ingest_client import IcsAnalytics
from factories import configuration_factory, dtc_factory, message_factory


class TestAnalyticsDTCIntegration(unittest.TestCase):
    def setUp(self):
        self.config_data = configuration_factory()
        self.client = IcsAnalytics(
            device_id=self.config_data['device_id'],
            vehicle_id=self.config_data['vehicle_id'],
            fleet_id=self.config_data['fleet_id'],
            org_id=self.config_data['organization_id'],
            batch_size=10,
            graphql_endpoint="http://0.0.0.0:8092/graphql",
        )

    def build_test_entry(self, dtc_data, message_data):
        return {
            "configurationId": 1,
            "fileId": "File_8",
            "messageId": "Msg_123",
            "messageDate": "2025-06-05T03:07:07Z",
            "data": dtc_data,
            "messageName": message_data["name"],
            "networkName": message_data["networkName"],
            "name": message_data["name"],
            **{k: v for k, v in message_data.items() if k not in ["name"]},
        }

    def test_add_dtc_valid(self):
        dtc_obj = dtc_factory(num_dtc_entries=1, items_per_dtc=5)[0]
        message_data = message_factory(self.config_data['vehicle_id'])[0]
        entry = self.build_test_entry(dtc_obj["data"], message_data)

        try:
            self.client.add_dtc([entry])
        except Exception as e:
            self.fail(f"Valid input raised unexpected error: {e}")

    def test_add_dtc_missing_description(self):
        dtc_obj = dtc_factory(num_dtc_entries=1, items_per_dtc=3)[0]
        del dtc_obj["data"][0]["description"]
        message_data = message_factory(self.config_data['vehicle_id'])[0]
        entry = self.build_test_entry(dtc_obj["data"], message_data)

        with self.assertRaises(Exception) as context:
            self.client.add_dtc([entry])
        self.assertIn("description", str(context.exception).lower())

    def test_add_dtc_invalid_time_format(self):
        dtc_obj = dtc_factory(num_dtc_entries=1, items_per_dtc=1)[0]
        dtc_obj["data"][0]["time"] = "not-a-time"
        message_data = message_factory(self.config_data['vehicle_id'])[0]
        entry = self.build_test_entry(dtc_obj["data"], message_data)

        with self.assertRaises(Exception) as context:
            self.client.add_dtc([entry])
        self.assertIn("time", str(context.exception).lower())

    def test_add_dtc_empty_data_list(self):
        message_data = message_factory(self.config_data['vehicle_id'])[0]
        entry = self.build_test_entry([], message_data)

        try:
            self.client.add_dtc([entry])
        except Exception as e:
            self.fail(
                f"Empty DTC data list should not raise an exception, but got: {e}"
            )

    def test_add_dtc_missing_data_key(self):
        message_data = message_factory(self.config_data['vehicle_id'])[0]
        entry = self.build_test_entry(None, message_data)
        entry.pop("data", None)

        try:
            self.client.add_dtc([entry])
        except Exception as e:
            self.fail(f"Missing 'data' key should not raise exception, but got: {e}")



import os
from typing import Optional

from pydantic import BaseModel, Field, model_validator


class IngestConfigSchema(BaseModel):
    device_id: int
    vehicle_id: int
    fleet_id: int
    org_id: int

    batch_interval_seconds: Optional[int] = Field(
        default=10, ge=1, description="Must be >= 1"
    )
    batch_size: Optional[int] = Field(default=100, ge=1, description="Must be >= 1")
    graphql_endpoint: str = Field(default_factory=lambda: os.getenv("GRAPH_ENDPOINT"))
    max_signal_count: Optional[int] = Field(default=100, ge=1)

    @model_validator(mode="after")
    def validate_env_or_param(self):
        if not self.graphql_endpoint:
            raise ValueError(
                "Missing GraphQL endpoint. Set via param or GRAPH_ENDPOINT env."
            )
        return self



import asyncio

import requests
from graphql import parse, print_ast
from graphql.error import GraphQLSyntaxError
from requests.exceptions import RequestException


class GraphQLExecutor:
    def __init__(self, graphql_endpoint):
        self.graphql_endpoint = graphql_endpoint
        self._semaphore = asyncio.Semaphore(1)

    async def execute_async(self, query: str, variables: dict = None):
        async with self._semaphore:
            return self._execute(query, variables)

    def _execute(self, query: str, variables: dict = None):
        try:
            parsed_query = print_ast(parse(query))
            headers = {}

            request_data = {"query": parsed_query, "variables": variables}
            print("request data", request_data)
            response = requests.post(
                self.graphql_endpoint, json=request_data, headers=headers
            )

            if not response.text:
                raise RuntimeError("Empty response received from GraphQL endpoint.")

            response_data = response.json()
            if "errors" in response_data:
                raise RuntimeError(
                    f"GraphQL request failed with errors: {response_data['errors']}"
                )
            res = {"data": response_data["data"]}
            print("response ===> ", res)
            return res

        except (RequestException, GraphQLSyntaxError) as e:
            raise RuntimeError(f"Invalid GraphQL syntax: {e}")

    def execute(self, query: str, variables: dict = None):
        return self._execute(query, variables)



from datetime import datetime

from analytics_ingest.internal.schemas.dtc_schema import DTCSchema
from analytics_ingest.internal.schemas.inputs.dtc_input import make_dtc_input
from analytics_ingest.internal.schemas.message_schema import MessageSchema
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.message import get_cached_message_id
from analytics_ingest.internal.utils.mutations import GraphQLMutations
from analytics_ingest.internal.utils.serialization import serialize_payload


def create_dtc(
    executor: GraphQLExecutor,
    config_id: str,
    variables_list: list[dict],
    batch_size: int,
):
    all_messages_with_dtcs = {}

    for variables in variables_list:
        file_id = variables.get("fileId", "")
        message_date = variables.get("messageDate", "")
        message = MessageSchema.from_variables(variables)
        message_key = message.cache_key()
        message_id = get_cached_message_id(message_key)

        if not message_id:
            raise RuntimeError(f"No message ID found for key: {message_key}")

        dtc_items = DTCSchema.from_variables(variables)
        batches = Batcher.create_batches(dtc_items, batch_size)

        for batch in batches:
            if message_id not in all_messages_with_dtcs:
                all_messages_with_dtcs[message_id] = {
                    "fileId": file_id,
                    "messageId": message_id,
                    "messageDate": message_date,
                    "dtcs": [],
                }
            all_messages_with_dtcs[message_id]["dtcs"].extend(
                [item.model_dump() for item in batch]
            )

    sorted_dtcs_data = [
        {
            "configurationId": config_id,
            "fileId": str(msg["fileId"]),
            "messageId": msg["messageId"],
            "messageDate": msg["messageDate"],
            "data": msg["dtcs"],
        }
        for msg in sorted(
            all_messages_with_dtcs.values(),
            key=lambda x: datetime.fromisoformat(x["messageDate"]),
        )
    ]

    payload = serialize_payload(make_dtc_input(sorted_dtcs_data))
    executor.execute(GraphQLMutations.upsert_dtc_mutation(), payload)


def build_batched_dtc_inputs(
    config_id, variables, message_id, fileId, messageDate, batch_size
):
    dtc_items = DTCSchema.from_variables(variables)
    batches = Batcher.create_batches(dtc_items, batch_size)
    return [
        make_dtc_input(config_id, batch, message_id, fileId, messageDate)
        for batch in batches
    ]



import json
from datetime import datetime


def serialize_payload(payload):
    return json.loads(
        json.dumps(
            payload, default=lambda o: o.isoformat() if isinstance(o, datetime) else o
        )
    )



# analytics_ingest/src/analytics_ingest/internal/schemas/gps_schema.py

from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class AvailableSchema(BaseModel):
    accuracy: Optional[bool] = None
    altitude: Optional[bool] = None
    bearing: Optional[bool] = None
    speed: Optional[bool] = None
    time: Optional[bool] = None


class GPSSchema(BaseModel):
    time: datetime
    latitude: Optional[float] = None
    longitude: Optional[float] = None
    accuracy: Optional[float] = None
    altitude: Optional[float] = None
    speed: Optional[float] = None
    bearing: Optional[str] = None
    available: Optional[AvailableSchema] = None

    @classmethod
    def from_variables(cls, variables: dict) -> list["GPSSchema"]:
        data = variables.get("data", [])
        if not data:
            raise ValueError("Missing required field: 'data'")
        return [cls(**item) for item in data]



class Batcher:
    @staticmethod
    def create_batches(data: list, batch_size: int) -> list:
        if not isinstance(data, list):
            raise TypeError(f"'data' must be a list, got {type(data).__name__}")
        if not isinstance(batch_size, int) or batch_size <= 0:
            raise ValueError(
                f"'batch_size' must be a positive integer, got {batch_size}"
            )
        return [data[i : i + batch_size] for i in range(0, len(data), batch_size)]



from analytics_ingest.internal.schemas.inputs.message_input import (
    make_create_message_input,
)
from analytics_ingest.internal.schemas.message_schema import MessageSchema
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.mutations import GraphQLMutations

_message_cache = {}


def get_cached_message_id(key: str) -> str | None:
    print("Create Key:", key)
    return _message_cache.get(key)


def create_message(executor: GraphQLExecutor, variables: list[dict]) -> list[str]:
    message_ids = []
    uncached_messages = []
    comparison_keys = []

    for var in variables:
        key = generate_message_cache_key(var)

        if key in _message_cache:
            message_ids.append(_message_cache[key])
        else:
            input_dict = {
                "arbId": var.get("arbId"),
                "name": var.get("name"),
                "networkName": var.get("networkName"),
                "ecuName": var.get("ecuName"),
                "ecuId": var.get("ecuId"),
                "requestCode": var.get("requestCode"),
                "fileId": var.get("fileId"),
                "messageDate": var.get("messageDate"),
            }
            comparison_keys.append(key)
            uncached_messages.append(input_dict)

    if uncached_messages:
        response = executor.execute(
            GraphQLMutations.create_message(),
            make_create_message_input(uncached_messages),
        )
        if "errors" in response:
            error_message = f"Error in create_message response: {response['errors']}"
            raise RuntimeError("Error in create_message_response: %s", error_message)

        messages = response["data"].get("createMessage", [])
        if not messages:
            raise RuntimeError("No messages created")

        for idx, input_dict in enumerate(uncached_messages):
            key = comparison_keys[idx]
            matching = next(
                (
                    m
                    for m in messages
                    if all(
                        m.get(k) == input_dict.get(k)
                        for k in input_dict.keys()
                        if k != "requestCode"
                    )
                ),
                None,
            )
            if matching:
                _message_cache[key] = str(matching["id"])
                message_ids.append(str(matching["id"]))

    return message_ids


def generate_message_cache_key(data: dict) -> str:
    """Consistent key used for message caching and lookup."""
    return f"{data.get('arbId')}|{data.get('networkName')}|{data.get('ecuName')}|{data.get('name')}"



import time
from threading import Thread
from typing import List

from analytics_ingest.internal.schemas.signal_schema import SignalSchema
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.message import (
    create_message,
    get_cached_message_id,
    generate_message_cache_key,
)


class SignalBufferManager:
    def __init__(
        self,
        executor,
        configuration_id: int,
        batch_size: int,
        max_signal_count: int,
        batch_interval_seconds: int = 10,
    ):
        self.executor = executor  # Still needed for message creation
        self.configuration_id = configuration_id
        self.batch_size = batch_size
        self.max_signal_count = max_signal_count
        self.batch_interval_seconds = batch_interval_seconds

        self.buffer: List[dict] = []
        self.last_flush_time = time.time()

        self._start_background_flush()

    def _start_background_flush(self):
        thread = Thread(target=self._background_flush_loop, daemon=True)
        thread.start()

    def _background_flush_loop(self):
        while True:
            time.sleep(1)
            if self.buffer and (
                time.time() - self.last_flush_time >= self.batch_interval_seconds
            ):
                print("[BackgroundFlush] Time-based flush triggered")
                try:
                    self.flush(force=True)
                except Exception as e:
                    print(f"[SignalBufferManager] Flush error: {e}")

    def add_signal(self, signals: List[dict]):
        if not signals:
            print("[add_signal] No signals received")
            return

        print(f"[add_signal] Received {len(signals)} signals")

        try:
            print("[add_signal] Creating messages...")
            create_message(self.executor, signals)
            print("[add_signal] Messages created")
        except Exception as e:
            print(f"[add_signal] Failed to create messages: {e}")
            return

        self.buffer.extend(signals)
        print(f"[add_signal] Buffer size after add: {len(self.buffer)}")

        self.flush()

    def flush(self, force=False):
        if not self.buffer:
            print("[flush] No signals to flush")
            return

        print(f"[flush] Flushing {len(self.buffer)} signals (force={force})")

        signals_to_flush = self.buffer
        self.buffer = []
        self.last_flush_time = time.time()

        batches = Batcher.create_batches(signals_to_flush, self.batch_size)
        print(f"[flush] Created {len(batches)} batches")

        for idx, batch in enumerate(batches):
            print(f"[flush] Processing batch {idx + 1} with {len(batch)} signals")

            for signal in batch:
                message_key = generate_message_cache_key(signal)
                message_id = get_cached_message_id(message_key)
                print(f"[flush] message_key={message_key}, message_id={message_id}")

                if message_id is None:
                    print(f"[flush] WARNING: Missing message_id for key {message_key}")
                    continue

                signal_input = SignalSchema.from_variables(
                    self.configuration_id,
                    int(message_id),
                    signal["data"],
                    signal,
                )
                print(f"[flush] Signal prepared: {signal_input.model_dump()}")

        print(f"[flush] (Mock) All signals processed. No actual GraphQL request made.")

    def _chunk_signals(self, signals: List[dict]) -> List[List[dict]]:
        return [
            signals[i : i + self.max_signal_count]
            for i in range(0, len(signals), self.max_signal_count)
        ]



from typing import List, Optional

from pydantic import BaseModel


class SignalSchema(BaseModel):
    configurationId: int
    messageId: int
    name: str
    unit: str
    paramType: Optional[str] = ""
    paramId: Optional[str] = ""
    signalType: Optional[str] = ""
    data: List[dict]

    @classmethod
    def from_variables(cls, config_id, message_id, batch, variables):
        return cls(
            configurationId=config_id,
            messageId=message_id,
            name=variables["name"],
            paramId=variables.get("paramId", ""),
            paramType=variables.get("paramType", ""),
            signalType=variables.get("signalType", ""),
            unit=variables["unit"],
            data=batch,
        )



from pydantic import BaseModel


class ConfigurationSchema(BaseModel):
    deviceId: int
    fleetId: int
    organizationId: int
    vehicleId: int

    @classmethod
    def from_variables(cls, variables: dict) -> "ConfigurationSchema":
        return cls(
            deviceId=variables["device_id"],
            fleetId=variables["fleet_id"],
            organizationId=variables["org_id"],
            vehicleId=variables["vehicle_id"],
        )



from analytics_ingest.internal.schemas.inputs.signal_input import make_signal_input
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.mutations import GraphQLMutations


def build_batched_signal_inputs(config_id, variables, message_id, batch_size):
    if "data" not in variables:
        raise ValueError("Missing required field: 'data'")
    batches = Batcher.create_batches(variables["data"], batch_size)
    return [
        make_signal_input(config_id, batch, message_id, variables) for batch in batches
    ]


def create_signal(
    executor: GraphQLExecutor,
    config_id: str,
    variables: dict,
    message_id: str,
    batch_size: int,
):
    inputs = build_batched_signal_inputs(config_id, variables, message_id, batch_size)
    for payload in inputs:
        executor.execute(GraphQLMutations.upsert_signal_data(), payload)



import time
import unittest
from unittest.mock import MagicMock, patch

from analytics_ingest.ingest_client import IcsAnalytics
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.mutations import GraphQLMutations
from factories import configuration_factory, message_factory, signal_factory


class TestAnalyticsIngestClient(unittest.TestCase):
    def setUp(self):
        self.config = configuration_factory()
        with patch(
            "analytics_ingest.ingest_client.SignalBufferManager"
        ) as MockBufferManager:
            self.mock_buffer_manager = MockBufferManager.return_value
            self.client = IcsAnalytics(
                device_id=self.config['device_id'],
                vehicle_id=self.config['vehicle_id'],
                fleet_id=self.config['fleet_id'],
                org_id=self.config['organization_id'],
                graphql_endpoint="http://0.0.0.0:8092/graphql",
                batch_size=5,
                batch_interval_seconds=2,
            )

    @patch("analytics_ingest.internal.utils.message.create_message")
    def test_add_signal_valid_and_flushed(self, mock_create_message):
        signals = [
            s
            for _ in range(10)
            for s in signal_factory(vehicle_id=self.config["vehicle_id"])
        ]

        self.client.add_signal(signals)

        self.client.signal_buffer_manager.flush()

        self.mock_buffer_manager.add_signal.assert_called_with(signals)
        self.mock_buffer_manager.flush.assert_called()

    def test_buffer_flush_on_time(self):
        signals = [
            signal_factory(vehicle_id=self.config["vehicle_id"])[0] for _ in range(3)
        ]
        self.client.add_signal(signals)

        self.client.signal_buffer_manager.flush()

        self.mock_buffer_manager.flush.assert_called()


if __name__ == "__main__":
    unittest.main()



import random
from datetime import datetime, timedelta

from faker import Faker

fake = Faker()


def network_stats_factory(vehicle_id: int) -> dict:
    base_time = datetime.utcnow()

    return {
        "errorMessages": random.randint(1, 200),
        "longMessageParts": random.randint(1, 200),
        "matchedMessages": random.randint(100, 200),
        "maxTime": (base_time + timedelta(minutes=random.randint(1, 60))).isoformat()
        + "Z",
        "minTime": (base_time - timedelta(minutes=random.randint(1, 60))).isoformat()
        + "Z",
        "name": fake.domain_word(),
        "rate": round(random.uniform(1.0, 100.0), 2),
        "totalMessages": random.randint(200, 500),
        "unmatchedMessages": random.randint(0, 100),
        "uploadId": 4255,
        "vehicleId": vehicle_id,
    }



import unittest
from copy import deepcopy

from analytics_ingest.ingest_client import IcsAnalytics
from factories import configuration_factory, gps_factory, message_factory


class TestAnalyticsGPSIntegeration(unittest.TestCase):
    def setUp(self):
        self.config_data = configuration_factory()
        self.client = IcsAnalytics(
            device_id=self.config_data['device_id'],
            vehicle_id=self.config_data['vehicle_id'],
            fleet_id=self.config_data['fleet_id'],
            org_id=self.config_data['organization_id'],
            batch_size=10,
            graphql_endpoint="http://0.0.0.0:8092/graphql",
        )

    def test_add_gps_signal_with_factories(self):
        test_variables = gps_factory(num_entries=10)
        config_id = self.client.configuration_id
        self.assertIsInstance(config_id, int)
        self.client.add_gps(test_variables)

    def test_add_gps_with_valid_manual_data(self):
        valid_entry = gps_factory(num_entries=1)["data"][0]
        gps_data = {"data": [valid_entry for _ in range(5)]}
        try:
            message_data = message_factory(self.config_data['vehicle_id'])[0]
            test_variables = {
                **message_data,
                **gps_data,
                "messageName": message_data["name"],
            }
            self.client.add_gps(test_variables)
        except Exception as e:
            self.fail(f"Valid input raised unexpected error: {e}")

    def test_add_gps_missing_time(self):
        bad_entry = deepcopy(gps_factory(num_entries=1)["data"][0])
        del bad_entry["time"]
        test_variables = {"data": [bad_entry]}
        with self.assertRaises(Exception) as context:
            self.client.add_gps(test_variables)
        self.assertIn("time", str(context.exception).lower())

    def test_add_gps_invalid_latitude_type(self):
        bad_entry = deepcopy(gps_factory(num_entries=1)["data"][0])
        bad_entry["latitude"] = "not-a-float"
        test_variables = {"data": [bad_entry]}
        with self.assertRaises(Exception) as context:
            self.client.add_gps(test_variables)
        self.assertIn("latitude", str(context.exception).lower())

    def test_add_gps_empty_data_list(self):
        test_variables = {"data": []}
        with self.assertRaises(RuntimeError) as context:
            self.client.add_gps(test_variables)
        self.assertIn("missing required field", str(context.exception).lower())

    def test_add_gps_missing_data_key(self):
        test_variables = {}
        with self.assertRaises(ValueError) as context:
            self.client.add_gps(test_variables)
        self.assertIn("missing 'variables'", str(context.exception).lower())



from typing import Optional

from pydantic import BaseModel


class MessageSchema(BaseModel):
    arbId: Optional[str] = ""
    name: str
    networkName: str = ""
    ecuName: Optional[str] = ""
    ecuId: Optional[str] = ""
    fileId: Optional[str] = ""
    messageDate: Optional[str] = ""
    requestCode: Optional[str] = ""

    @classmethod
    def from_variables(cls, variables):
        return cls(
            arbId=variables.get("arbId", ""),
            name=variables["messageName"],
            networkName=variables.get("networkName", ""),
            ecuName=variables.get("ecuName", ""),
            ecuId=variables.get("ecuId", ""),
            fileId=variables.get("fileId", ""),
            messageDate=variables.get("messageDate", ""),
            requestCode=variables.get("requestCode", ""),
        )

    def cache_key(self) -> str:
        return f"{self.name}|{self.networkName}|{self.ecuName}|{self.arbId}"



from analytics_ingest.internal.schemas.configuration_schema import ConfigurationSchema
from analytics_ingest.internal.schemas.inputs.configuration_input import (
    make_configuration_input_from_schema,
)
from analytics_ingest.internal.utils.mutations import GraphQLMutations


class ConfigurationService:
    def __init__(self, executor):
        self.executor = executor

    def create(self, config_dict):
        schema = ConfigurationSchema.from_variables(config_dict)
        return self.executor.execute(
            GraphQLMutations.create_configuration(),
            make_configuration_input_from_schema(schema),
        )



import random
from datetime import datetime, timedelta, timezone

from faker import Faker

fake = Faker()


def random_hex_status():
    return f"{random.randint(0, 255):02X}"


def single_dtc_item(base_time, index_offset):
    entry_time = (base_time + timedelta(seconds=index_offset)).isoformat()

    entry = {
        "dtcId": f"P{random.randint(1000, 9999)}-{random.randint(10, 99)}",
        "value": random.choice(["ACTIVE", "PASSIVE"]),
        "status": random_hex_status(),
        "description": fake.sentence(nb_words=6),
        "time": entry_time,
        "extension": [],
        "snapshot": [],
    }

    if random.choice([True, False]):
        entry["extension"].append({"bytes": fake.hexify(text="^" * 8)})

    if random.choice([True, False]):
        entry["snapshot"].append({"bytes": fake.hexify(text="^" * 8)})

    return entry


def dtc_factory(num_dtc_entries=1000, items_per_dtc=50):
    base_time = datetime.now(timezone.utc)
    dtc_data = []

    for dtc_index in range(num_dtc_entries):
        dtc_items = [
            single_dtc_item(base_time, dtc_index * items_per_dtc + i)
            for i in range(items_per_dtc)
        ]

        dtc_data.append({"data": dtc_items})

    return dtc_data



import asyncio
import time
import unittest
from unittest.mock import MagicMock, patch

from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor


class TestGraphQLExecutorConcurrency(unittest.TestCase):
    def setUp(self):
        self.executor = GraphQLExecutor("http://mock/graphql")

    @patch("analytics_ingest.internal.utils.graphql_executor.requests.post")
    def test_concurrent_requests_execute_serially(self, mock_post):
        def delayed_post(*args, **kwargs):
            time.sleep(0.5)
            mock_response = MagicMock()
            mock_response.text = '{"data": {"ok": true}}'
            mock_response.json.return_value = {"data": {"ok": True}}
            return mock_response

        mock_post.side_effect = delayed_post

        request_count = 5
        execution_times = []

        async def run_query(i):
            start = time.monotonic()
            await self.executor.execute_async("query { dummy }")
            end = time.monotonic()
            execution_times.append((i, start, end))

        async def main():
            tasks = [run_query(i) for i in range(request_count)]
            await asyncio.gather(*tasks)

        asyncio.run(main())

        self.assertEqual(mock_post.call_count, request_count)

        for i in range(1, len(execution_times)):
            prev_end = execution_times[i - 1][2]
            curr_start = execution_times[i][1]
            self.assertGreaterEqual(
                curr_start,
                prev_end,
                f"Request {i} started before previous finished — Semaphore failed",
            )

        total_duration = execution_times[-1][2] - execution_times[0][1]
        self.assertGreaterEqual(total_duration, 0.5 * request_count - 0.1)


if __name__ == "__main__":
    unittest.main()



class GraphQLMutations:
    @staticmethod
    def create_configuration():
        return """
            mutation createConfiguration($input: CreateConfigurationInput!) {
                createConfiguration(input: $input) {
                    id
                    deviceId
                    vehicleId
                    organizationId
                    fleetId
                }
            }
        """

    @staticmethod
    def create_message():
        return """
             mutation createMessage($input: CreateMessageInput!) {
                createMessage(input: $input) {
                    id
                    arbId
                    name
                    networkName
                    ecuId
                    ecuName
                    fileId
                }
            }
        """

    @staticmethod
    def upsert_signal_data():
        return """
            mutation UpsertSignalData($input: UpsertSignalDataInput) {
                upsertSignalData(input: $input) {
                    configurationId
                    messageId
                    messageName
                    name
                    paramType
                    unit
                }
            }
        """

    @staticmethod
    def upsert_gps_data():
        return """
             mutation UpsertGpsData($input: UpsertGpsDataInput) {
                upsertGpsData(input: $input) {
                    deviceId
                    fleetId
                    vehicleId
                    id
                    organizationId
                }
            }
        """

    @staticmethod
    def create_network_stats_mutation():
        return """
            mutation createNetworkStats($input: CreateNetworkStatsInput!) {
                createNetworkStats(input: $input) {
                    name
                    vehicleId
                    uploadId
                    totalMessages
                    matchedMessages
                    unmatchedMessages
                    errorMessages
                    longMessageParts
                    minTime
                    maxTime
                    rate
                }
            }
        """

    @staticmethod
    def upsert_dtc_mutation():
        return """
            mutation UpsertDtcData($input: [UpsertDtcDataInput]) {
                upsertDtcData(input: $input) {
                    configurationId
                    messageId
                }
            }
        """



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel


class BytesInput(BaseModel):
    bytes: str


class DTCSchema(BaseModel):
    dtcId: str
    status: str
    description: str
    time: datetime
    extended: Optional[List[BytesInput]] = None
    snapshot: Optional[List[BytesInput]] = None

    @classmethod
    def from_variables(cls, variables: dict) -> list["DTCSchema"]:
        return [cls(**item) for item in variables.get("data", [])]



import logging

from faker import Faker

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

fake = Faker()


def message_factory(vehicle_id):
    return [
        {
            "name": f"TestMessage_{vehicle_id}",
            "messageName": f"TestMessage_{vehicle_id}",
            "networkName": "CAN",
            "arbId": f"0x{fake.random_int(min=100, max=999):03X}",
            "ecuId": f"ECU{fake.random_int(min=1, max=9)}",
            "messageDate": fake.date_time_between(
                start_date="-1y", end_date="now"
            ).strftime("%Y-%m-%dT%H:%M:%SZ"),
            "requestCode": f"Code_{fake.word()}",
            "ecuName": f"ECU_{fake.word()}",
            "fileId": f"File_{fake.random_int(min=1, max=9)}",
        }
    ]



from analytics_ingest.internal.schemas.ingest_config_schema import IngestConfigSchema


def init_schema_factory(**overrides):
    base = {
        "device_id": 1,
        "vehicle_id": 2,
        "fleet_id": 3,
        "org_id": 4,
        "graphql_endpoint": "http://0.0.0.0:8092/graphql",
        "batch_size": 50,
        "batch_interval_seconds": 5,
    }
    base.update(overrides)
    return base



from analytics_ingest.internal.schemas.signal_schema import SignalSchema


def make_signal_input(config_id, batch, message_id, variables):
    signal = SignalSchema.from_variables(config_id, message_id, batch, variables)
    return {"input": {"signals": [signal.model_dump()]}}



import asyncio
import time
from collections import deque
from threading import Thread
from typing import Optional

from more_itertools import chunked

from analytics_ingest.internal.schemas.ingest_config_schema import IngestConfigSchema
from analytics_ingest.internal.schemas.message_schema import MessageSchema
from analytics_ingest.internal.schemas.signal_schema import SignalSchema
from analytics_ingest.internal.utils.configuration import ConfigurationService
from analytics_ingest.internal.utils.dtc import create_dtc
from analytics_ingest.internal.utils.gps import create_gps
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.message import (
    create_message,
    get_cached_message_id,
)
from analytics_ingest.internal.utils.mutations import GraphQLMutations
from analytics_ingest.internal.utils.network import create_network
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.signal_buffer_manager import SignalBufferManager


class IcsAnalytics:
    def __init__(self, **kwargs):
        self.config = IngestConfigSchema(**kwargs)
        self.executor = GraphQLExecutor(self.config.graphql_endpoint)

        self.configuration_id = ConfigurationService(self.executor).create(
            self.config.model_dump()
        )["data"]["createConfiguration"]["id"]

        self.signal_buffer_manager = SignalBufferManager(
            executor=self.executor,
            configuration_id=self.configuration_id,
            batch_size=self.config.batch_size,
            max_signal_count=self.config.max_signal_count,
            batch_interval_seconds=self.config.batch_interval_seconds,
        )

    def add_signal(self, signals: Optional[list] = None):
        if not isinstance(signals, list):
            raise ValueError("'signals' should be a list of dicts")
        if not signals:
            raise ValueError("Missing 'signals' list")

        try:
            self.signal_buffer_manager.add_signal(signals)
        except Exception as e:
            logger.error(f"Failed to add signals: {e}")

    def add_dtc(self, variables_list: Optional[list] = None):
        if not variables_list:
            raise ValueError("Missing 'dtc' list")
        try:
            create_message(self.executor, variables_list)
            create_dtc(
                executor=self.executor,
                config_id=self.configuration_id,
                variables_list=variables_list,
                batch_size=self.config.batch_size,
            )
        except Exception as e:
            raise RuntimeError(f"Failed to add DTC: {e}")

    def add_gps(self, variables: Optional[dict] = None):
        if not variables:
            raise ValueError("Missing 'variables' dictionary")
        try:
            create_gps(
                executor=self.executor,
                config_id=self.configuration_id,
                variables=variables,
                batch_size=self.config.batch_size,
            )
        except Exception as e:
            raise RuntimeError(f"Failed to add GPS: {e}")

    def add_network_stats(self, variables: Optional[dict] = None):
        if not variables:
            raise ValueError("Missing 'variables' dictionary")
        try:
            create_network(
                executor=self.executor,
                config=self.config,
                variables=variables,
            )
        except Exception as e:
            raise RuntimeError(f"Failed to add network stats: {e}")

    def close(self):
        self._shutdown = True

        if self.signal_buffer:
            future = asyncio.run_coroutine_threadsafe(self._flush_buffer(), self.loop)
            try:
                future.result(timeout=10)
            except Exception as e:
                print(f"Final flush failed: {e}")

        if self.loop.is_running():
            self.loop.call_soon_threadsafe(self.loop.stop)

        if hasattr(self, "thread") and self.thread.is_alive():
            self.thread.join(timeout=2)



import unittest

from analytics_ingest.ingest_client import IcsAnalytics
from analytics_ingest.internal.schemas.configuration_schema import ConfigurationSchema
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from factories import configuration_factory, network_stats_factory


class TestAnalyticsNetworkStatsIntegration(unittest.TestCase):
    def setUp(self):
        self.config_data = configuration_factory()
        self.client = IcsAnalytics(
            device_id=self.config_data['device_id'],
            vehicle_id=self.config_data['vehicle_id'],
            fleet_id=self.config_data['fleet_id'],
            org_id=self.config_data['organization_id'],
            batch_size=10,
            graphql_endpoint="http://0.0.0.0:8092/graphql",
        )

    def test_add_network_stats_valid(self):
        data = network_stats_factory(self.config_data['vehicle_id'])
        try:
            self.client.add_network_stats(data)
        except Exception:
            self.fail("add_network_stats() raised Exception unexpectedly!")

    def test_add_network_stats_invalid_type(self):
        data = network_stats_factory(self.config_data['vehicle_id'])
        data["rate"] = "not_a_float"
        with self.assertRaises(RuntimeError) as cm:
            self.client.add_network_stats(data)
        self.assertIn("Failed to add network stats", str(cm.exception))

    def test_add_network_stats_missing_variables(self):
        with self.assertRaises(ValueError) as cm:
            self.client.add_network_stats(None)
        self.assertEqual(str(cm.exception), "Missing 'variables' dictionary")


if __name__ == "__main__":
    unittest.main()



from analytics_ingest.internal.schemas.inputs.network_input import make_network_input
from analytics_ingest.internal.schemas.network_schema import NetworkStatsSchema
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.mutations import GraphQLMutations


def create_network(executor: GraphQLExecutor, config, variables: dict):
    payload = make_network_input(config, variables)
    executor.execute(GraphQLMutations.create_network_stats_mutation(), payload)


def create_network_payload(config, variables):
    schema = NetworkSchema.from_variables(variables)
    return make_network_input(config, schema)



import logging
from datetime import datetime, timedelta

from faker import Faker

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
from factories import message_factory

fake = Faker()


def signal_factory(vehicle_id):
    units = ["RPM", "km/h", "m/s", "kPa", "degC"]
    param_types = ["NUMBER", "STRING", "TEXT"]
    signal_types = ["SIGNAL", "DID", "PID", "DMR"]
    message = message_factory(vehicle_id=vehicle_id)[0]

    return [
        {
            "name": f"Signal_{fake.word()}_{fake.random_int(min=1000, max=9999)}",
            "unit": fake.random_element(units),
            "paramType": fake.random_element(param_types),
            "signalType": fake.random_element(signal_types),
            "paramId": f"param_{fake.random_int(min=1000, max=9999)}",
            "messageName": message["name"],
            "ecuId": message["ecuId"],
            "arbId": message["arbId"],
            "messageDate": message["messageDate"],
            "fileId": message["fileId"],
            "requestCode": message["requestCode"],
            "networkName": message["networkName"],
            "ecuName": message["ecuName"],
            "data": [
                {
                    "value": fake.pyfloat(
                        min_value=1000, max_value=10000000, right_digits=1
                    ),
                    "time": (
                        fake.date_time_between(start_date="-1d", end_date="now")
                        + timedelta(seconds=i)
                    ).strftime("%Y-%m-%dT%H:%M:%SZ"),
                }
                for i in range(10)
            ],
        }
    ]



from analytics_ingest.internal.schemas.gps_schema import GPSSchema
from analytics_ingest.internal.schemas.inputs.gps_input import make_gps_input
from analytics_ingest.internal.utils.batching import Batcher
from analytics_ingest.internal.utils.graphql_executor import GraphQLExecutor
from analytics_ingest.internal.utils.mutations import GraphQLMutations
from analytics_ingest.internal.utils.serialization import serialize_payload


def create_gps(
    executor: GraphQLExecutor, config_id: str, variables: dict, batch_size: int
):
    inputs = build_batched_gps_inputs(config_id, variables, batch_size)
    for payload in inputs:
        payload = serialize_payload(payload)
        executor.execute(GraphQLMutations.upsert_gps_data(), payload)


def build_batched_gps_inputs(config_id, variables, batch_size):
    gps_items = GPSSchema.from_variables(variables)
    batches = Batcher.create_batches(gps_items, batch_size)
    return [make_gps_input(config_id, batch) for batch in batches]



import os
import unittest

from analytics_ingest.ingest_client import IcsAnalytics
from factories.init_schema_factory import init_schema_factory


class TestAnalyticsIngestClientInit(unittest.TestCase):
    def test_init_with_valid_config(self):
        config_data = init_schema_factory()
        client = IcsAnalytics(**config_data)

        self.assertEqual(client.config.device_id, config_data["device_id"])
        self.assertIsNotNone(client.executor)
        self.assertIsInstance(client.configuration_id, int)

    def test_missing_each_required_param(self):
        required_keys = ["device_id", "vehicle_id", "fleet_id", "org_id"]
        for key in required_keys:
            with self.subTest(key=key):
                config_data = init_schema_factory()
                config_data.pop(key)
                with self.assertRaises(ValueError) as ctx:
                    IcsAnalytics(**config_data)
                self.assertIn(key, str(ctx.exception))

    def test_invalid_type_for_int_fields(self):
        int_fields = ["device_id", "vehicle_id", "fleet_id", "org_id"]
        for key in int_fields:
            with self.subTest(key=key):
                config_data = init_schema_factory(**{key: "not-an-int"})
                with self.assertRaises(ValueError) as ctx:
                    IcsAnalytics(**config_data)
                self.assertIn(key, str(ctx.exception))

    def test_missing_graphql_endpoint_raises_when_not_in_env(self):
        config_data = init_schema_factory(graphql_endpoint=None)
        if "GRAPH_ENDPOINT" in os.environ:
            del os.environ["GRAPH_ENDPOINT"]

        with self.assertRaises(ValueError) as ctx:
            IcsAnalytics(**config_data)

        self.assertIn("graphql_endpoint", str(ctx.exception))

    def test_config_schema_object_is_attached(self):
        config_data = init_schema_factory()
        client = IcsAnalytics(**config_data)
        self.assertEqual(client.config.device_id, config_data["device_id"])

    def test_executor_is_set(self):
        config_data = init_schema_factory()
        client = IcsAnalytics(**config_data)
        self.assertIsNotNone(client.executor)

    def test_invalid_batch_size_negative(self):
        config_data = init_schema_factory(batch_size=-5)
        with self.assertRaises(ValueError) as ctx:
            IcsAnalytics(**config_data)
        self.assertIn("batch_size", str(ctx.exception))

    from unittest.mock import patch

    @patch("analytics_ingest.ingest_client.GraphQLExecutor.execute")
    def test_env_fallbacks(self, mock_execute):
        os.environ["SEC_AUTH_TOKEN"] = "env-token"
        os.environ["GRAPH_ENDPOINT"] = "http://env-graphql"

        mock_execute.return_value = {"data": {"createConfiguration": {"id": 123}}}

        config_data = init_schema_factory()
        config_data.pop("graphql_endpoint")

        client = IcsAnalytics(**config_data)

        self.assertEqual(client.config.graphql_endpoint, "http://env-graphql")



from typing import Optional

from pydantic import BaseModel


class NetworkStatsSchema(BaseModel):
    name: str
    vehicleId: int
    uploadId: int
    totalMessages: int
    matchedMessages: int
    unmatchedMessages: int
    errorMessages: int
    longMessageParts: int
    maxTime: Optional[str] = None
    minTime: Optional[str] = None
    rate: float

    @classmethod
    def from_variables(cls, variables: dict, vehicle_id: int) -> "NetworkStatsSchema":
        return cls(
            errorMessages=variables["errorMessages"],
            longMessageParts=variables["longMessageParts"],
            matchedMessages=variables["matchedMessages"],
            maxTime=variables.get("maxTime"),
            minTime=variables.get("minTime"),
            name=variables["name"],
            rate=variables["rate"],
            totalMessages=variables["totalMessages"],
            unmatchedMessages=variables["unmatchedMessages"],
            uploadId=variables["uploadId"],
            vehicleId=vehicle_id,
        )



import random
from datetime import datetime, timedelta, timezone

from faker import Faker

fake = Faker()


def gps_factory(num_entries=120):
    base_time = datetime.now(timezone.utc)
    gps_data = []

    for i in range(num_entries):

        timestamp = (base_time + timedelta(seconds=i)).isoformat()
        entry = {
            "time": timestamp,
            "latitude": float(round(fake.latitude(), 6)),
            "longitude": float(round(fake.longitude(), 6)),
            "accuracy": float(round(random.uniform(5.0, 50.0), 2)),
            "altitude": float(round(random.uniform(100.0, 1000.0), 2)),
            "speed": float(round(random.uniform(0.0, 120.0), 2)),
            "available": {
                "accuracy": bool(random.getrandbits(1)),
                "altitude": bool(random.getrandbits(1)),
                "bearing": bool(random.getrandbits(1)),
                "date": bool(random.getrandbits(1)),
                "latlon": bool(random.getrandbits(1)),
                "speed": bool(random.getrandbits(1)),
                "time": bool(random.getrandbits(1)),
            },
        }

        gps_data.append(entry)

    return {"data": gps_data}
